/*
 * console_led.c
 */

#include "cons_led.h"

#include <stdlib.h>
#include <stdio.h>


/* shell command prototypes */
void console_cmd_ledprog_help(int argc, char **argv);
void console_cmd_ledprog_list(int argc, char **argv);
void console_cmd_ledprog_change(int argc, char **argv);
void console_cmd_ledprog_link(int argc, char **argv);
void console_cmd_ledprog_enable(int argc, char **argv);
void console_cmd_ledprog_disable(int argc, char **argv);
void console_cmd_ledprog_delete(int argc, char **argv);
void console_cmd_ledprog_exitmode(int argc, char **argv);

/* shell commands */
static uint8_t cons_led_mode;
static const command_table_t console_ledprog_cmdlist[] = {
		{"help",    "this text, '?' works too; also try 'help <cmd>'",	0x00, console_cmd_ledprog_help},
		{"?", 	    "",						                            0x00, console_cmd_ledprog_help},
		{"list",    "print table of LED programs",						0x00, console_cmd_ledprog_list},
		{"change",  "add or update a program index",					0x00, console_cmd_ledprog_change},
		{"lnk",  	"link a step to an existing program index",			0x00, console_cmd_ledprog_link},
		{"enable",  "enable specified program index",					0x00, console_cmd_ledprog_enable},
		{"disable", "disable specified program index",					0x00, console_cmd_ledprog_disable},
		{"delete",  "delete specified program index",					0x00, console_cmd_ledprog_delete},
		{"remove",  "",													0x00, console_cmd_ledprog_delete},
		{"exit",    "leave LED program edit function mode",				0x00, console_cmd_ledprog_exitmode},
		{"done",    "",													0x00, console_cmd_ledprog_exitmode},
		{"quit",    "",													0x00, console_cmd_ledprog_exitmode},
		{"bye",     "",													0x00, console_cmd_ledprog_exitmode},
		{NULL, NULL, 0x00, NULL}
};


void console_cmd_led_bclock(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Enables or disables the binary clock.\r\n");
		CONS_PRINT("Why would anyone disable this wizardy?\r\n");
	}

	if (argc == 1) {
		CONS_PRINT("Usage: bclock <1/0>     (note: 1 is the only real option.)\r\n");
	}

	if (argc == 2) {
		if (argv[1][0] == '1') { 			// enable
			led_disable &= ~(1 << LED_CLOCK_IDX);
		} else if (argv[1][0] == '0') { 	// disable
			led_disable |= (1 << LED_CLOCK_IDX);
			CONS_PRINT("whyyyy? nooooo");
		} else { 							// dumb
			CONS_PRINT("* error: learn to binary");
		}
	}
}

/* enter subconsole */
void console_cmd_led_eye(int argc, char **argv)
{
	cons_led_mode = LED_EYES_IDX;
	ctbl_prev = ctbl;
	ctbl = (command_table_t *)console_ledprog_cmdlist;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_LED_EYE);
}

void console_cmd_led_skull(int argc, char **argv)
{
	cons_led_mode = LED_SKULL_IDX;
	ctbl_prev = ctbl;
	ctbl = (command_table_t *)console_ledprog_cmdlist;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_LED_SKULL);
}

void console_cmd_led_bone(int argc, char **argv)
{
	cons_led_mode = LED_BONE_IDX;
	ctbl_prev = ctbl;
	ctbl = (command_table_t *)console_ledprog_cmdlist;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_LED_BONE);
}

/* subconsole callbacks */
void console_cmd_ledprog_help(int argc, char **argv)
{
	console_help(argc, argv, (command_table_t *)console_ledprog_cmdlist);
}

void console_cmd_ledprog_list(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Prints a nicely formatted table of programs. ");
		CONS_PRINT("Just run the command.");
		return;
	}

	int i;
	LEDProg *ledprog = 0;

	switch (cons_led_mode) {
		case LED_SKULL_IDX: 	ledprog = (LEDProg *)prog_skull; break;
		case LED_EYES_IDX: 		break;
		case LED_BONE_IDX:  	break;
		default: 				return;
	}

	CONS_PRINT("/----------------------------------------------------------\\");
	CONS_PRINT("| ID | Program Name      | Speed | Options           | Ena |");

	for (i = 0; i < LED_PROG_COUNT; i++) {
		if (ledprog[i].flags & LED_PROG_ENABLED) {
			sprintf(console_line, "| %02d | %*s | %03dms | %08x %08x | %*s",
					(i + 1),
					-17, "todo",
					ledprog[i].wait,
					(unsigned int)ledprog[i].optn[0], (unsigned int)ledprog[i].optn[1],
					-3, (ledprog[i].flags & LED_PROG_ENABLED) ? "Yes" : "no.");
		}
	}

	CONS_PRINT("\\----------------------------------------------------------/");
}

void console_cmd_ledprog_change(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Creates or edits an LED program.\r\n");
		CONS_PRINT("Run 'change <idx>' and answer the questions.");
		return;
	}
}

void console_cmd_ledprog_link(int argc, char **argv)
{

}

void console_cmd_ledprog_enable(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Enables a program to run. Enabled programs run sequentially.\r\n");
		CONS_PRINT("Specify a two-digit program ID to enable.");
		return;
	}
}

void console_cmd_ledprog_disable(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Disables a program. Enabled programs run sequentially.\r\n");
		CONS_PRINT("Specify a two-digit program ID to disable.");
		return;
	}
}

void console_cmd_ledprog_delete(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Deletes a program. No confirmation.\r\n");
		CONS_PRINT("Specify a two-digit program ID to delete.");
		return;
	}
}

void console_cmd_ledprog_exitmode(int argc, char **argv)
{
	ctbl = ctbl_prev;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_DEFAULT);
}
