/*
 * console_time.c
 * ----
 * there's surely a better way to do most of this.
 */

#include "cons_timedate.h"

#include <stdlib.h>


/* working alarm */
static RTCAlarm *walarm;
static uint8_t walarm_id;
static uint8_t walarm_state;


/* shell command prototypes */
void console_cmd_time_alarm_help(int argc, char **argv);
void console_cmd_time_alarm_list(int argc, char **argv);
void console_cmd_time_alarm_new(int argc, char **argv);
void console_cmd_time_alarm_edit_time(int argc, char **argv);
void console_cmd_time_alarm_edit_name(int argc, char **argv);
void console_cmd_time_alarm_enable(int argc, char **argv);
void console_cmd_time_alarm_disable(int argc, char **argv);
void console_cmd_time_alarm_delete(int argc, char **argv);
void console_cmd_time_alarm_exitmode(int argc, char **argv);

/* shell commands */
static const command_table_t console_time_alarm_cmdlist[] = {
		{"help",    "this text, '?' works too; also try 'help <cmd>'",	0x00, console_cmd_time_alarm_help},
		{"?", 	    "",						                            0x00, console_cmd_time_alarm_help},
		{"list",    "print table of alarms",							0x00, console_cmd_time_alarm_list},
		{"new",     "create a new alarm (up to 16 alarms)",				0x00, console_cmd_time_alarm_new},
		{"edit",    "edit an existing alarm activation time",			0x00, console_cmd_time_alarm_edit_time},
		{"name", 	"change the name of an alarm", 						0x00, console_cmd_time_alarm_edit_name},
		{"enable",  "arm an alarm",										0x00, console_cmd_time_alarm_enable},
		{"disable", "disarm an alarm",									0x00, console_cmd_time_alarm_disable},
		{"delete",  "remove an alarm",									0x00, console_cmd_time_alarm_delete},
		{"remove",  "",													0x00, console_cmd_time_alarm_delete},
		{"exit",    "leave alarm function mode",						0x00, console_cmd_time_alarm_exitmode},
		{"done",    "",													0x00, console_cmd_time_alarm_exitmode},
		{"quit",    "",													0x00, console_cmd_time_alarm_exitmode},
		{"bye",     "",													0x00, console_cmd_time_alarm_exitmode},
		{NULL, NULL, 0x00, NULL}
};


/* root shell time shit */
void console_cmd_time_time(int argc, char **argv)
{
	// print help
	if (!argc) {
		CONS_PRINT("Prints the time. If arguments are supplied, sets the time.\r\n\r\n");
		CONS_PRINT("    time <hour, 00-23> <minute, 00-59> <second, 00-59>\r\n\r\n");
		CONS_PRINT("Example: 'time 13 37 00' will set the time to 1:37PM\r\n");
		return;
	}

	// set clock
	if (argc == 4) {
		// fuck it we don't sanitize
		rtc_set_time(
				(((argv[1][0] - 0x30) * 10) + argv[1][1] - 0x30),
				(((argv[2][0] - 0x30) * 10) + argv[2][1] - 0x30),
				(((argv[3][0] - 0x30) * 10) + argv[3][1] - 0x30)
		);
	}

	strcpy(console_line, rtc_print_time());
	strcat(console_line, "\r\n");
	CONS_PRINT(console_line);
}

void console_cmd_time_date(int argc, char **argv)
{
	if (!argc) {
		// print help
		CONS_PRINT("Prints the date. If arguments are supplied, sets the date.\r\n\r\n");
		CONS_PRINT("    date <month, 01-12> <day, 01-31>\r\n\r\n");
		CONS_PRINT("Example: 'date 08 06' set the date to August 6th.\r\n");
		CONS_PRINT("Note: Only 'Murrica format supported. All months 31 days.\r\n");
		return;
	}

	// set clock
	if (argc == 3) {
		// fuck it we don't sanitize
		rtc_set_date(
				(((argv[1][0] - 0x30) * 10) + argv[1][1] - 0x30),
				(((argv[2][0] - 0x30) * 10) + argv[2][1] - 0x30)
		);
	}

	strcpy(console_line, rtc_print_date());
	strcat(console_line, "\r\n");
	CONS_PRINT(console_line);
}

void console_cmd_time_alarm_vibrate(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Configures the alarm vibrator pattern.\r\n");
		CONS_PRINT("Run 'vibrate test <X>' where X is an ID 1-4 to test.\r\n");
		CONS_PRINT("Run 'vibrate set <X>' where X is an ID 1-4 to activate.\r\n");
	}

	int idx;

	if (argc == 3) {
		idx = atoi(argv[2]);
		if (idx && idx <= 4) {
			if (strcmp(argv[1], "set") == 0) {
				return;
			} else if (strcmp(argv[1], "test") == 0) {
				return;
			}
		}
	}

	CONS_PRINT("Usage: vibrate <test/set> <digit 1-4>");
}


/* alarm subshell */
void console_cmd_time_alarm(int argc, char **argv)
{
	ctbl_prev = ctbl;
	ctbl = (command_table_t *)console_time_alarm_cmdlist;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_TIME_ALARM);
}

void console_cmd_time_alarm_help(int argc, char **argv)
{
	console_help(argc, argv, (command_table_t *)console_time_alarm_cmdlist);
}

void console_cmd_time_alarm_list(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Prints a nicely formatted table of alarms. ");
		CONS_PRINT("Just run the command.");
		return;
	}

	int i;
	RTCAlarm *a;

	CONS_PRINT("/--------------------------------------------------\\\r\n");
	CONS_PRINT("| ID | Name                    | When        | Ena |\r\n");
	CONS_PRINT("|--------------------------------------------------|\r\n");

	for (i = 0; i < RTC_ALARM_COUNT; i++) {
		a = alarm_getptr(i);
		if (a->modeflags & RTC_ALARM_MODE_CREATED) {
			sprintf(console_line, "| %02d | %*s | %*s %02d:%02d | %*s |\r\n",
					i + 1,
					-23, a->name,
					-5, alarm_get_modetext(i), a->hour, a->minute,
					-3, (a->modeflags & RTC_ALARM_MODE_ENABLE) ? "Yes" : "no.");
			CONS_PRINT(console_line);
		}
	}

	CONS_PRINT("\\--------------------------------------------------/\r\n");
}

int console_cmd_time_alarm_new_cb(const char *text, void *extobj)
{
	uint16_t time;
	char w[4];

	switch (walarm_state) {
		case 0: { 	// name
			if (text[0] == 0) {
				CONS_PRINT("! ERR Name is empty, aborting.");
				console_hijack(0);
				return -2;
			}
			if (strlen(text) > 23) {
				CONS_PRINT("! ERR Name too long, try again:      ");
				return -1;
			}

			// seems legit
			strcpy(walarm->name, text);

			// might as well clear this here
			walarm->actflags = 0;

			walarm_state++;
			CONS_PRINT("* Enter the trigger hour, 0-23:      ");
			break;
		}
		case 1: { 	// hour
			time = atoi(text);
			if (time > 23) {
				CONS_PRINT("! ERR WTF numbers??, try again 0-23: ");
				return -1;
			}

			// seems legit
			walarm->hour = time & 0xff;

			walarm_state++;
			CONS_PRINT("* Enter the trigger minute, 0-59:    ");
			break;
		}
		case 2: { 	// minute
			time = atoi(text);
			if (time > 59) {
				CONS_PRINT("! ERR WTF numbers??, try again 0-59: ");
				return -1;
			}

			// seems legit
			walarm->minute = time & 0xff;

			// might as well clear this here
			walarm->modeflags &= ~RTC_ALARM_MODE_MASK;

			walarm_state++;
			CONS_PRINT("* Trigger [d]ay, [o]nce or [r]epeat? ");
			break;
		}
		case 3: { 	// alarm type
			switch (text[0]) {
				case 'd':
				case 'D': {
					walarm_state++;
					CONS_PRINT("* Enter month and day, MMDD format:  ");
					return 0;
				}

				case 'o':
				case 'O': {
					walarm->modeflags |= RTC_ALARM_MODE_ONESHOT;
					break;
				}

				case 'r':
				case 'R': {
					walarm->modeflags |= RTC_ALARM_MODE_REPEAT;
					break;
				}
				default: {
					CONS_PRINT("! ERR not [d], [o], [r]; try again:  ");
					return -1;
				}
			}

			walarm_state += 2;
			CONS_PRINT("* Enable the alarm now? [yn]         ");
			break;
		}
		case 4: { 	// month and day
			w[0] = text[0]; w[1] = text[1];
			time = atoi(w);
			if (time && time <= 12) {
				walarm->month = time;
			} else {
				CONS_PRINT("! ERR bad month supplied, new MMDD:  ");
				return -1;
			}

			w[0] = text[2]; w[1] = text[3];
			time = atoi(w);
			if (time && time <= 31) {
				walarm->day = time;
			} else {
				CONS_PRINT("! ERR bad day supplied, new MMDD:    ");
				return -1;
			}

			walarm_state++;
			CONS_PRINT("* Enable the alarm now? [yn]         ");
			break;
		}
		case 5: { 	// enable
			switch (text[0]) {
				case 'y':
				case 'Y': {
					walarm->modeflags |= RTC_ALARM_MODE_ENABLE;
					break;
				}
				case 'n':
				case 'N': {
					break;
				}
				default: {
					CONS_PRINT("! ERR It's either [y]es or [n]o...:  ");
					return 0;
				}
			}

			// copy alarm info
			sprintf(console_line, "Alarm %s at ID %02d.",
					(walarm->modeflags & RTC_ALARM_MODE_CREATED) ? "updated" : "created",
					walarm_id + 1);

			walarm->modeflags |= RTC_ALARM_MODE_CREATED;

			CONS_PRINT(console_line);
			console_hijack(0);
			break;
		}
		default: {
			CONS_PRINT("! Unknown error.");
			console_hijack(0);
		}
	}

	return 0;
}

void console_cmd_time_alarm_new(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Creates a new alarm in the first available slot.\r\n");
		CONS_PRINT("Run 'new' and answer the questions to create the alarm.");
		return;
	}

	int i;
	RTCAlarm *a;

	// find first ID
	for (i = 0; i < RTC_ALARM_COUNT; i++) {
		a = alarm_getptr(i);
		if ((a->modeflags & RTC_ALARM_MODE_CREATED) == 0) {
			// this is an inactive alarm, so create our shit here
			walarm = a;
			walarm_id = i;
			walarm_state = 0;

			console_hijack(console_cmd_time_alarm_new_cb);
			CONS_PRINT("* Alarm name (village, track, etc):  ");
			return;
		}
	}

	CONS_PRINT("* all alarm slots in use, cannot create new alarm");
}

void console_cmd_time_alarm_edit_time(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Prompts to edit the time and date/mode of an alarm.\r\n");
		CONS_PRINT("Specify a two-digit alarm ID to edit.");
		return;
	}

	if (argc == 1) {
		CONS_PRINT("Usage: edit <01-16>\r\n");
		return;
	}

	uint16_t idx;

	if (argc == 2) {
		idx = atoi(argv[1]);
		if (idx && idx <= RTC_ALARM_COUNT) {
			walarm = alarm_getptr(idx - 1);
			if (walarm->modeflags & RTC_ALARM_MODE_CREATED) {
				walarm_id = idx - 1;
				walarm_state = 1;
				console_hijack(console_cmd_time_alarm_new_cb);

				CONS_PRINT("* Enter the trigger hour, 0-23:      ");
			} else {
				CONS_PRINT("* this alarm ID doesn't exist");
			}
		}
	}
}

int console_cmd_time_alarm_edit_name_cb(const char *text, void *extobj)
{
	if (text[0] == 0) {
		CONS_PRINT("! ERR Name is empty, aborting.");
		console_hijack(0);
		return -2;
	}
	if (strlen(text) > 23) {
		CONS_PRINT("! ERR Long, retry: ");
		return -1;
	}

	// seems legit
	strcpy(walarm->name, text);

	sprintf(console_line, "* Alarm ID %02d name updated.", walarm_id + 1);
	CONS_PRINT(console_line);

	console_hijack(0);
	return 0;
}

void console_cmd_time_alarm_edit_name(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Prompts to edit the name of an alarm.\r\n");
		CONS_PRINT("Specify a two-digit alarm ID to edit.");
		return;
	}

	if (argc == 1) {
		CONS_PRINT("Usage: name <01-16>\r\n");
		return;
	}

	uint16_t idx;

	if (argc == 2) {
		idx = atoi(argv[1]);
		if (idx && idx <= RTC_ALARM_COUNT) {
			walarm = alarm_getptr(idx - 1);
			if (walarm->modeflags & RTC_ALARM_MODE_CREATED) {
				walarm_id = idx - 1;
				console_hijack(console_cmd_time_alarm_edit_name_cb);

				sprintf(console_line, "Alarm %02d old name: %s\r\n", idx, walarm->name);
				CONS_PRINT(console_line);
				CONS_PRINT("New (max 23 char): ");
			} else {
				CONS_PRINT("* this alarm ID doesn't exist");
			}
		}
	}
}

void console_cmd_time_alarm_enable(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Enables an alarm. Specify the two-digit alarm ID ");
			CONS_PRINT("to enable as an argument.");
		return;
	}

	if (argc == 1) {
		CONS_PRINT("Usage: enable <01-16>\r\n");
		return;
	}

	uint16_t id;

	if (argc == 2) {
		id = atoi(argv[1]);
		if (id && id <= RTC_ALARM_COUNT) {
			alarm_enable(id - 1, 1);
			return;
		}
	}

	CONS_PRINT("* bad ID or some other error");
}

void console_cmd_time_alarm_disable(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Disables an alarm. Specify the two-digit alarm ID ");
			CONS_PRINT("to disable as an argument.");
		return;
	}

	if (argc == 1) {
		CONS_PRINT("Usage: disable <01-16>\r\n");
		return;
	}

	uint16_t id;

	if (argc == 2) {
		id = atoi(argv[1]);
		if (id && id <= RTC_ALARM_COUNT) {
			alarm_enable(id - 1, 0);
			return;
		}
	}

	CONS_PRINT("* bad ID or some other error");
}

void console_cmd_time_alarm_delete(int argc, char **argv)
{
	if (!argc) {
		CONS_PRINT("Deletes an alarm. Specify the two-digit alarm ID ");
			CONS_PRINT("to delete as an argument.");
		return;
	}

	if (argc == 1) {
		CONS_PRINT("Usage: delete <01-16>\r\n");
		return;
	}

	uint16_t id;

	if (argc == 2) {
		id = atoi(argv[1]);
		if (id && id <= RTC_ALARM_COUNT) {
			alarm_delete((id & 0xff) - 1);
			CONS_PRINT("* if it was there, this alarm ID is now deleted");
			return;
		}
	}

	CONS_PRINT("* could not delete: invalid ID or something");
}

void console_cmd_time_alarm_exitmode(int argc, char **argv)
{
	CONS_PRINT("To commit changes to flash, run 'save alarm'.\r\n");
	ctbl = ctbl_prev;
	ntshell_set_prompt(&ntshell, CONS_PROMPT_DEFAULT);
}

void console_time_alarm_save()
{
	CONS_PRINT("Saving alarms...");
	if (alarm_save_nv() != SPIFLASH_OK) {
		CONS_PRINT("failed.\r\n");
		return;
	}
	CONS_PRINT("done.\r\n");
}
