/**
 * led.h
 */

#ifndef __WP2016_DISPLAY_LED_H
#define __WP2016_DISPLAY_LED_H


#include <stdint.h>


/* programs */
typedef struct {
	uint8_t  idx;
	uint8_t flags;
	uint16_t wait;
	uint32_t init;
	uint32_t optn[2];
} LEDProg;

#define LED_PROG_COUNT 		20		// 240 bytes per mode

#define LED_PROG_ENABLED 	0x0001


/* led levels */
extern uint16_t led_eyes[2][3];
extern uint16_t led_skull[8];
extern uint16_t led_bone[5][3];
extern uint16_t led_clock[16];
extern uint8_t  led_flags;

extern uint8_t led_disable;

/* led scale values */
extern uint16_t led_pwm_max;

extern const uint8_t led_eyes_scalemax[5];


/* led shiftreg / disable state */
#define LED_EYES_IDX 		0
#define LED_SKULL_IDX 		1
#define LED_BONE_IDX 		2
#define LED_CLOCK_IDX 		3

/* led general defines */
#define LED_BRT_MAX 		9999

#define LED_RED 			0
#define LED_GREEN 			1
#define LED_BLUE 			2

#define LED_GREEN_RIGHT 	2
#define LED_BLUE_RIGHT 		1


/* shift register */
#define LED_EYES_MASK 		0x000e
#define LED_EYES_SHIFT 		1
#define LED_SKULL_MASK 		0x0271
#define LED_SKULL_SHIFT_HI 	3
#define LED_BONE_MASK 		0x7c00
#define LED_BONE_SHIFT 		10


/* led pwm indices */
#define LED_PWM_SKULL_0 	0
#define LED_PWM_CLOCK_1 	1
#define LED_PWM_CLOCK_0 	2
#define LED_PWM_SKULL_1 	3
#define LED_PWM_EYE_LEFT 	4
#define LED_PWM_EYE_RIGHT 	5
#define LED_PWM_BONE_BLUE 	6
#define LED_PWM_BONE_RED 	7
#define LED_PWM_BONE_GREEN 	8


/* prototypes */
void led_init();
void led_stop();

void led_state_next();


/* rendering */
void led_render_eyes();
void led_render_skull();
void led_render_bone();
void led_render_clock();


#endif
