/**
 * adc.c
 * ------
 * implements ADC single mode
 */


#include "adc.h"


/* data and callback */
static void (*adc_cb[ADC_SINGLE_MAX_CHANNELS])(uint16_t result);
static uint16_t adc_result[ADC_SINGLE_MAX_CHANNELS];


/* setup */
void adc_init()
{
	ADC_InitSingle_TypeDef adc_init = ADC_INITSINGLE_DEFAULT;

	adc_init.reference = ADC_VREF_AVDD;
	adc_init.input = ADC_INPUT_LIGHTSENS;
	ADC_InitSingle(ADC0, &adc_init);

	ADC_IntClear(ADC0, ADC_IFC_SINGLE);
	ADC_IntEnable(ADC0, ADC_IF_SINGLE);

	NVIC_SetPriority(ADC0_IRQn, 4);
	NVIC_EnableIRQ(ADC0_IRQn);
}

void adc_cb_init(uint8_t channel, void (*cb)(uint16_t result))
{
	if (channel < ADC_SINGLE_MAX_CHANNELS) {
		adc_cb[channel] = cb;
	}
}

void adc_start()
{
	ADC_Start(ADC0, adcStartSingle);
}

/* utilities */
void adc_set_chan(uint8_t input)
{
	ADC0->SINGLECTRL &= _ADC_SINGLECTRL_INPUTSEL_MASK;
	ADC0->SINGLECTRL |= (input << _ADC_SINGLECTRL_INPUTSEL_SHIFT);
}

void adc_set_vref(uint8_t vref)
{
	ADC0->SINGLECTRL &= _ADC_SINGLECTRL_REF_MASK;
	ADC0->SINGLECTRL |= (vref << _ADC_SINGLECTRL_REF_SHIFT);
}

uint16_t adc_getresult(uint8_t chan)
{
	if (chan >= ADC_SINGLE_MAX_CHANNELS) {
		return 0;
	}

	return adc_result[chan];
}

/* isr */
void ADC0_IRQHandler()
{
	uint32_t flags;
	uint8_t  idx;

	flags = ADC_IntGet(ADC0);

	if (flags & ADC_IF_SINGLE) {
		ADC_IntClear(ADC0, ADC_IFC_SINGLE);

		idx = (ADC0->SINGLECTRL & _ADC_SINGLECTRL_INPUTSEL_MASK) >> _ADC_SINGLECTRL_INPUTSEL_SHIFT;

		if (idx < ADC_SINGLE_MAX_CHANNELS) {
			adc_result[idx] = ADC0->SINGLEDATA;
			if (adc_cb[idx] != 0) {
				adc_cb[idx](adc_result[idx]);
			}
		}
	}
}
