/**
 * usb.c
 */

#include "em_device.h"
#include "em_chip.h"

#include "usbconfig.h"
#include "descriptors.h"
#include "cdc.h"

#include "em_usb.h"


/* prototypes */
static void USB_StateChange(USBD_State_TypeDef oldState,
                        USBD_State_TypeDef newState);


/* usb setup structs */
static const USBD_Callbacks_TypeDef callbacks =
{
	.usbReset        = NULL,
	.usbStateChange  = USB_StateChange,
	.setupCmd        = CDC_SetupCmd,
	.isSelfPowered   = NULL,
	.sofInt          = NULL
};

static const USBD_Init_TypeDef usb_init_struct =
{
	.deviceDescriptor    = &USBDESC_deviceDesc,
	.configDescriptor    = USBDESC_configDesc,
	.stringDescriptors   = USBDESC_strings,
	.numberOfStrings     = sizeof(USBDESC_strings)/sizeof(void*),
	.callbacks           = &callbacks,
	.bufferingMultiplier = USBDESC_bufferingMultiplier,
	.reserved            = 0
};


/* setup */
void usb_init()
{
	// set pins
	GPIO->P[2].MODEH = (GPIO->P[2].MODEH & ~_GPIO_P_MODEH_MODE14_MASK)
			| GPIO_P_MODEH_MODE14_DISABLED;
	GPIO->P[2].MODEH = (GPIO->P[2].MODEH & ~_GPIO_P_MODEH_MODE15_MASK)
			| GPIO_P_MODEH_MODE15_DISABLED;

	USBD_Init(&usb_init_struct);
}

void usb_stop()
{
	USBD_Stop();
}


/* state change handler */
static void USB_StateChange(USBD_State_TypeDef oldState,
                        USBD_State_TypeDef newState)
{
	CDC_StateChangeEvent(oldState, newState);
}


/* helpers */
int usb_is_configured()
{
	if (USBD_GetUsbState() == USBD_STATE_CONFIGURED) return 1;
	return 0;
}
